#!/usr/bin/python
# -*- python -*-

#
# gfx.py -- main script of pymp3 and main class for screen handling
#
# Copyright (c) 2004 DEGOIS Damien <damien@degois.info> 
# Licensed under the GNU GPL. For full terms see the file COPYING.
#
# Cf LICENSE provided with those files
#

import curses, traceback, string, os, sys
import layer
import sound_ctrl
import box_info


version = "0.3.1"

hotkey_attr = curses.A_BOLD | curses.A_UNDERLINE
menu_attr = curses.A_NORMAL

screen = None

class NavWin:
    def __init__(self):
        self.cur_loc = os.getcwd()
        self.dir_cur = 0
        self.first_page_cur = 0
        self.size_of_list = 0
        self.lets_quit = 0
        self.dir_cont = 0
        self.stdscr = 0
        self.top_margin = 7
        # Clasik
        curses.init_pair(1,curses.COLOR_WHITE,curses.COLOR_BLACK)
        # Clasik surligned
        curses.init_pair(2,curses.COLOR_WHITE,curses.COLOR_BLUE)
        # Dir 
        curses.init_pair(3,curses.COLOR_GREEN,curses.COLOR_BLACK)
        # Dir Surligned
        curses.init_pair(4,curses.COLOR_GREEN,curses.COLOR_BLUE)
        # MP3
        curses.init_pair(5,curses.COLOR_RED,curses.COLOR_BLACK)
        # MP3 Surligned
        curses.init_pair(6,curses.COLOR_RED,curses.COLOR_BLUE)
        

        
    def showdir(self):
        """ showdir affiche le contenu du rep """
        dir_content = os.listdir(self.cur_loc)
        dir_content.append("..")
        dir_content.sort()
        dir_content.insert(1,".")
        return dir_content

    def design_screen(self):
        """ Entoure l'ecran d'un filet blanc et dessigne une ligne en haut """
        screen.box()
        screen.hline(self.top_margin - 1, 1, curses.ACS_HLINE, self.maxx-2)
        screen.refresh()

        
    def run(self,stdscr):
        """ Processus de base """
        global screen
        (self.maxy, self.maxx) = stdscr.getmaxyx()
        screen = stdscr.subwin(self.maxy, self.maxx, 0, 0)
        global showinfo
        showinfo = box_info.Info(screen,lecteur,mixer,version,"Shower")
        self.size_of_list = self.maxy-self.top_margin-1
        screen.keypad(1)
        self.design_screen()

        while not self.lets_quit :
            self.show_file_list()
            c = screen.getch()
            self.keyh(c)
        lecteur.stack.flush()

    def show_file_list(self):
        """ Affiche la liste des fichiers du rep """
        self.dir_cont = self.showdir()
        # Draw the onscreen field titles
        for x in range(self.first_page_cur, self.first_page_cur+self.size_of_list):
            if len(self.dir_cont) > x:
                suf = ""
                def_col = curses.color_pair(1)
                hl_col = curses.color_pair(2)
                if os.path.isdir(os.path.join(self.cur_loc, self.dir_cont[x])):
                    suf = "/"
                    def_col = curses.color_pair(3)
                    hl_col = curses.color_pair(4)
                (shortname, extension) = os.path.splitext(os.path.join(self.cur_loc, self.dir_cont[x]))
                if extension == ".mp3" or extension == ".MP3":
                    def_col = curses.color_pair(5)
                    hl_col = curses.color_pair(6)
                
                txt = "  "+self.dir_cont[x]+suf
                if len(txt) > self.maxx-4:
                    txt = txt[0:(self.maxx-5)]+"..."
                txt = txt + " "*(self.maxx-2-len(txt))
                if x == self.dir_cur:
                    screen.addstr(self.top_margin+x-(self.first_page_cur), 1, txt, curses.A_BOLD | hl_col)
                else:
                    screen.addstr(self.top_margin+x-(self.first_page_cur), 1, txt, curses.A_BOLD | def_col)


    def clean_window(self):
        """ Netoye l'ecran et redessine la ligne """
        for x in range (self.top_margin,self.maxy-1):
            screen.addstr(x,1," "*(self.maxx-2))
        self.design_screen()

    def ext_is_mp3(self,file):
        (shortname, extension) = os.path.splitext(file)
        if extension == ".mp3" or extension == ".MP3":
            return True
        else:
            return False


    def keyh(self,key):
        """ Fonction de prise en charge du clavier """
        # screen.addstr(1, self.maxx-5, "    ", curses.A_BOLD)
        # screen.addstr(1, self.maxx-5, str(key), curses.A_BOLD)

        # Q => Quit
        if key == 113:
            self.lets_quit = 1
        # Fleche bas : descend le curseur et gere les pages
        if key == curses.KEY_DOWN:
            if self.dir_cur >= len(self.dir_cont)-1: return
            if self.dir_cur >= (self.first_page_cur+self.size_of_list-2) and len(self.dir_cont) > self.size_of_list:
                self.first_page_cur = self.size_of_list-2+self.first_page_cur
                self.clean_window()
            self.dir_cur = self.dir_cur+1
        #Fleche heut : monte le curseur et gere les pages
        if key == curses.KEY_UP:
            if self.dir_cur == 0: return
            if self.dir_cur <= (self.first_page_cur+1):
                self.first_page_cur = self.first_page_cur-self.size_of_list+2
                if self.first_page_cur < 0: self.first_page_cur = 0
                self.clean_window()
            self.dir_cur = self.dir_cur-1

        #Pge Up
        if key == curses.KEY_PPAGE:
            if self.first_page_cur > 0:
                self.first_page_cur = self.first_page_cur - self.size_of_list
                self.dir_cur = self.dir_cur - self.size_of_list
                if self.first_page_cur < 0:
                    self.first_page_cur = 0
                if self.dir_cur < 0:
                    self.dir_cur = 0
        #Pge Down
        if key == curses.KEY_NPAGE and (self.size_of_list+self.dir_cur < len(self.dir_cont)):
            self.first_page_cur = self.first_page_cur + self.size_of_list
            self.dir_cur = self.dir_cur + self.size_of_list
            if self.first_page_cur+self.size_of_list > len(self.dir_cont)+1:
                self.first_page_cur = len(self.dir_cont)-2
                self.dir_cur = len(self.dir_cont) - 2
            if self.dir_cur > len(self.dir_cont):
                self.dir_cur = len(self.dir_cont)
            self.clean_window()

        # S => stop
        if key == 115:
            lecteur.orders.insert(0,"stop")

        # fleche droite
        if key == curses.KEY_RIGHT:
            lecteur.orders.insert(0,"fseek")

        # fleche gauche
        if key == curses.KEY_LEFT:
            lecteur.orders.insert(0,"bseek")

        # f => Flush
        if key == 102:
            lecteur.stack.flush()

        # r => Random
        if key == 114:
            if lecteur.random:
                lecteur.random = 0
            else:
                lecteur.random = 1
        
        # + => vol +
        if key == 43:
            mixer.orders.insert(0,"up")

        # - => vol -
        if key == 45:
            mixer.orders.insert(0,"down")

        if key == 110:
            lecteur.orders.insert(0,"next")

        # A => addpl :)
        if key == 97:
            lecteur.orders.insert(0,"addpl")
            lecteur.orders.insert(0,os.path.join(self.cur_loc, self.dir_cont[self.dir_cur]))


        if key == 10:
            if os.path.isdir(os.path.join(self.cur_loc, self.dir_cont[self.dir_cur])):
                old_loc = self.cur_loc
                self.cur_loc = os.path.abspath(os.path.join(self.cur_loc, self.dir_cont[self.dir_cur]))
                try:
                    os.listdir(self.cur_loc)
                except:
                    self.cur_loc = old_loc
                self.dir_cur = 0
                self.first_page_cur = 0
                self.clean_window()
            if self.ext_is_mp3(os.path.join(self.cur_loc, self.dir_cont[self.dir_cur])):
                lecteur.orders.insert(0,"addpl")
                lecteur.orders.insert(0,os.path.join(self.cur_loc, self.dir_cont[self.dir_cur]))
            
if __name__=='__main__':
    global lecteur
    lecteur = layer.MP3("Playa")
    global mixer
    mixer = sound_ctrl.Mixer("Mixa")
    try:
        # Initialize curses
        stdscr=curses.initscr()
        #curses.start_color()
        # Turn off echoing of keys, and enter cbreak mode,
        # where no buffering is performed on keyboard input
        curses.noecho() ; curses.cbreak()
        curses.start_color()

        # In keypad mode, escape sequences for special keys
        # (like the cursor keys) will be interpreted and
        # a special value like curses.KEY_LEFT will be returned
        stdscr.keypad(1)
        nw = NavWin()
        nw.run(stdscr)
        # Set everything back to normal
        if showinfo:
            showinfo.orders.insert(0,"exit")
            showinfo.join()
        stdscr.keypad(0)
        curses.echo() ; curses.nocbreak()
        curses.endwin()                 # Terminate curses
        lecteur.orders.insert(0,"stop")
        lecteur.orders.insert(0,"exit")
        mixer.orders.insert(0,"exit")
        print stdscr.getmaxyx()
        print lecteur.stack.size()
    except:
        # In the event of an error, restore the terminal
        # to a sane state.
        lecteur.stop()
        if showinfo:
            showinfo.orders.insert(0,"exit")
            showinfo.join()
        stdscr.keypad(0)
        curses.echo() ; curses.nocbreak()
        curses.endwin()
        traceback.print_exc()           # Print the exception
        print stdscr.getmaxyx()
        lecteur.orders.insert(0,"stop")
        lecteur.orders.insert(0,"exit")
        mixer.orders.insert(0,"exit")
